# -*- coding: utf8 -*-
import os
import re
import sys
import codecs
import shutil
import logging
from markdown import Markdown
from jinja2 import Template
from zipfile import ZipFile
from errno import EEXIST
from .data import codehilite_css, admonition_css, template_string

logging.basicConfig()


# Regex for markdown files
content_file_re = re.compile(r'[A-Za-z0-9\-_]+\.md')

# See the python-markdown documentation on the syntax provided by these
# markdown extensions
exts = ['markdown.extensions.' + ext
        for ext in ['extra', 'admonition', 'codehilite', 'headerid',
                    'meta', 'sane_lists']]


class A(object):
    "Contains the information give by dr's `a` file."
    def __init__(self, fname='a'):
        with codecs.open(fname, encoding='utf-8') as f:
            self.a = f.read()
        t = self.a.split('\n')
        if len(t) <= 2:
            logging.warn('You have less than 3 lines in your `a` file, you \
                    should probably go read how to write it on the xpose page')
        self.author = t[0]
        self.title = t[1]
        self.classifiers = t[2:]


def zipit(inpath='build/html', topath='build', title=None):
    "zips a directory to title.zip. Every file will be in the zipfile as\
            title/filename"
    with ZipFile(os.path.sep.join([topath, str(title) + '.zip']), 'w') as zipf:
        for fname in os.listdir(inpath):
            zipf.write(os.path.sep.join([inpath, fname]),
                       os.path.sep.join([str(title), fname]))


def slugify(s):
    """
    Slugify a string

    >>> slugify("Hello, I'm a slug!")
    hello_im_a_slug
    """
    pathed_title = s.lower().replace(' ', '_')
    return re.sub('[\'\"\?,:]', '', pathed_title)


def makedirs(path):
    "This is akin unix's `mkdir -p`"
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == EEXIST and os.path.isdir(path):
            pass
        else:
            raise


def write_css(to='build/html'):
    with codecs.open(os.path.sep.join([to, 'admonition.css']), 'w+') as f:
        f.write(admonition_css)
    with codecs.open(os.path.sep.join([to, 'codehilite.css']), 'w+') as f:
        f.write(codehilite_css)


def copy_ressources(frompath='content', to='build',
                    types=r'[a-zA-Z0-9_\-\.]+\.(^md)'):
    "Copy everything where the filename matches param types"
    for fname in os.listdir(frompath):
        if not re.match(".*\.(" + types + ")", fname):
            continue
        path = os.path.sep.join([frompath, fname])
        shutil.copy2(path, os.path.sep.join([to, fname]))


def _get_files_info(frompath):
    "parses and gets meta-information from every file matching `path`"
    md = Markdown(extensions=exts, output_format='html5')
    files = []
    for path in os.listdir(os.path.sep.join([os.getcwd(), frompath])):
        if not content_file_re.match(path):
            continue
        p = os.path.sep.join([os.getcwd(), "content", path])
        with codecs.open(p, encoding='utf-8') as f:
            content = f.read()
        html = md.convert(content)
        titles = []
        h2_re = re.compile(
            r'<h2 id="([a-z0-9\-]+)">([_a-zA-Z0-9\-\ \!\,]+)</h2>')
        for line in html.splitlines():
            match = h2_re.match(line)
            if match:
                titles.append({'id': match.group(1),
                               'title': match.group(2)})
        if 'title' not in md.Meta:
            logging.error("You forgot to set the Title meta-information in %s"
                          % path)
            sys.exit(1)
        fdict = {'path': path, 'title': md.Meta['title'][0],
                 'headers': titles, 'html': html,
                 'html_path': re.sub('\.md', '.html', path)}
        if "navorder" not in md.Meta:
            logging.warn("No NavOrder set for file %s. Navigation menu may be"
                         " erronous. See documentation." % fdict['path'])
        else:
            fdict.update({'order': md.Meta['navorder'][0]})
        files.append(fdict)
    return files


def gen_html(frompath='content', to='build/html'):
    "generate that shit. Returns the 'slugified' version of the site title."
    site_template = Template(template_string)
    files = sorted(_get_files_info(frompath),
                   key=lambda e: e['order'] if 'order' in e else e['title'])
    a = A()
    for d in files:
        real_html = site_template.render(site_title=a.title,
                                         page_title=d['title'],
                                         content=d['html'],
                                         navigation=files)
        path = d['path'].replace('.md', '.html')
        with codecs.open(os.path.sep.join([to, path]),
                         'w+',
                         encoding='utf-8') as f:
            f.write(real_html)
    return slugify(a.title)


def main():
    makedirs(os.path.sep.join(['build', 'html']))
    pathed_title = gen_html()
    copy_ressources()
    write_css(os.path.sep.join(['build', 'html']))
    shutil.copy2('a', os.path.sep.join(['build', 'html', 'a']))
    zipit(inpath=os.path.sep.join(['build', 'html']),
          topath='build',
          title=pathed_title)

if __name__ == '__main__':
    main()



codehilite_css = '#content .codehilite code,#content .codehilite pre{color:#fdce93;background-color:#3f3f3f;padding:1.5em;}.codehilite .hll{background-color:#222}.codehilite .c{color:#7f9f7f}.codehilite .err{color:#e37170;background-color:#3d3535}.codehilite .g{color:#7f9f7f}.codehilite .k{color:#f0dfaf}.codehilite .l{color:#ccc}.codehilite .n{color:#dcdccc}.codehilite .o{color:#f0efd0}.codehilite .x{color:#ccc}.codehilite .p{color:#41706f}.codehilite .cm{color:#7f9f7f}.codehilite .cp{color:#7f9f7f}.codehilite .c1{color:#7f9f7f}.codehilite .cs{color:#cd0000;font-weight:bold}.codehilite .gd{color:#cd0000}.codehilite .ge{color:#ccc;font-style:italic}.codehilite .gr{color:red}.codehilite .gh{color:#dcdccc;font-weight:bold}.codehilite .gi{color:#00cd00}.codehilite .go{color:gray}.codehilite .gp{color:#dcdccc;font-weight:bold}.codehilite .gs{color:#ccc;font-weight:bold}.codehilite .gu{color:purple;font-weight:bold}.codehilite .gt{color:#0040D0}.codehilite .kc{color:#dca3a3}.codehilite .kd{color:#ffff86}.codehilite .kn{color:#dfaf8f;font-weight:bold}.codehilite .kp{color:#cdcf99}.codehilite .kr{color:#cdcd00}.codehilite .kt{color:#00cd00}.codehilite .ld{color:#cc9393}.codehilite .m{color:#8cd0d3}.codehilite .s{color:#cc9393}.codehilite .na{color:#9ac39f}.codehilite .nb{color:#efef8f}.codehilite .nc{color:#efef8f}.codehilite .no{color:#ccc}.codehilite .nd{color:#ccc}.codehilite .ni{color:#c28182}.codehilite .ne{color:#c3bf9f;font-weight:bold}.codehilite .nf{color:#efef8f}.codehilite .nl{color:#ccc}.codehilite .nn{color:#8fbede}.codehilite .nx{color:#ccc}.codehilite .py{color:#ccc}.codehilite .nt{color:#9ac39f}.codehilite .nv{color:#dcdccc}.codehilite .ow{color:#f0efd0}.codehilite .w{color:#ccc}.codehilite .mf{color:#8cd0d3}.codehilite .mh{color:#8cd0d3}.codehilite .mi{color:#8cd0d3}.codehilite .mo{color:#8cd0d3}.codehilite .sb{color:#cc9393}.codehilite .sc{color:#cc9393}.codehilite .sd{color:#cc9393}.codehilite .s2{color:#cc9393}.codehilite .se{color:#cc9393}.codehilite .sh{color:#cc9393}.codehilite .si{color:#cc9393}.codehilite .sx{color:#cc9393}.codehilite .sr{color:#cc9393}.codehilite .s1{color:#cc9393}.codehilite .ss{color:#cc9393}.codehilite .bp{color:#efef8f}.codehilite .vc{color:#efef8f}.codehilite .vg{color:#dcdccc}.codehilite .vi{color:#ffffc7}.codehilite .il{color:#8cd0d3}'
admonition_css = 'div.admonition {margin-top: 10px;margin-bottom: 10px;padding: 7px;}div.admonition dt {font-weight: bold;}div.admonition dl {margin-bottom: 0;}p.admonition-title {margin: 0px 10px 5px 0px;font-weight: bold;}div.body p.centered {text-align: center;margin-top: 25px;}div.admonition, div.warning {font-size: 0.9em;margin: 1em 0 1em 0;border: 1px solid #86989B;border-radius: 2px;background-color: #f7f7f7;padding: 0;}div.admonition p, div.warning p {margin: 0.5em 1em 0.5em 1em;padding: 0;}div.admonition pre, div.warning pre {margin: 0.4em 1em 0.4em 1em;}div.admonition p.admonition-title,div.warning p.admonition-title {margin-top: 1em;padding-top: 0.5em;font-weight: bold;}div.warning {border: 1px solid #940000;}div.warning p.admonition-title {}div.admonition ul, div.admonition ol,div.warning ul, div.warning ol {margin: 0.1em 0.5em 0.5em 3em;padding: 0;}'
template_string = u'''
<!doctype html>
<html>
<head>
<title>{{site_title}}</title>
<link rel="stylesheet" type="text/css" title="Xposés" href="http://www-igm.\
univ-mlv.fr/~dr/XPOSE/style.css"/>
<link rel="stylesheet" type="text/css" href="admonition.css" />
<link rel="stylesheet" type="text/css" href="codehilite.css" />
<meta charset='utf-8' />
</head>
<body>
<div id="content">
<h1>{{site_title}}</h1>
{{content}}
</div>
<div id="navigation">
{% for page in navigation %}
<h2>{{page.title}}</h2>
<ul>
{% for header in page.headers %}
<li><a href={{page.html_path}}#{{header.id}}>{{header.title}}</a></li>
{% endfor %}
</ul>
{% endfor %}
</div>
'''
